// Copyright (c) 2006  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"

using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This is a template file for example programs.  \n"
  "The program itself does nothing whatsoever.    \n";

const string LongDescription =
  "Make a copy of this file (called \"foo.C\", say) and put your code \n"
  "inside the procedure \"program\".                                  \n"
  "To compile your file in the examples directory just do this:       \n"
  "  make foo                                                         \n";
//----------------------------------------------------------------------

namespace CoCoA
{

  long Regularity(const ideal& I)
  {
    Involutive::JBMill::Builder jBuilder;
    jBuilder.setInput(gens(I));

    Involutive::JBMill jbMill(jBuilder);

    if (jbMill.IamPommaretBasis())
    {
      Involutive::PBMill mill(Involutive::PBMill::Converter().setJBMill(jbMill));
      return mill.myRegularity();
    }
    // Ideal is not delta-regular
    Involutive::PBMill::DeltaRegularTransformator pBuilder;
    pBuilder.setInput(gens(I));
    pBuilder.setStrategy(Involutive::PBMill::DeltaRegularTransformator::SingleWithoutPermutation);
    pBuilder.setJanetStrategy(Involutive::GBCompletion);

    Involutive::PBMill mill(pBuilder);

    return mill.myRegularity();
  }

  void program()
  {
    GlobalManager CoCoAFoundations;

    cout << ShortDescription << endl;
    cout << boolalpha; // so that bools print out as true/false

    ring Q = RingQQ();
    SparsePolyRing P = NewPolyRing(Q, 3);

    const vector<RingElem>& x = indets(P);

    ideal I1 = ideal(x[0]*x[0],  x[1]);
    cout << "Regularity of I1:" << Regularity(I1) << endl;

    ideal I2 = ideal(x[1]*x[1],  x[2]);
    cout << endl << "Regularity of I2:" << Regularity(I2) << endl;

    ideal I3 = ideal(power(x[0],3),  power(x[1], 2), x[2]);
    cout << endl << "Regularity of I3:" << Regularity(I3) << endl;

    ideal I4 = ideal(power(x[1],3),  power(x[2], 2));
    cout << endl << "Regularity of I4:" << Regularity(I4) << endl;
  }

} // end of namespace CoCoA

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  // catch (const CoCoA::InterruptReceived& intr)
  // {
  //   cerr << "***WARNING***  UNHANDLED interrupt: " << endl
  //        << "*** " << intr << " ***" << endl;
  //   return 2;
  // }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}
